local Actions = include("sim/btree/actions")
local cdefs = include("client_defs")
local speechdefs = include( "sim/speechdefs" )
local simdefs = include( "sim/simdefs" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local simquery = include( "sim/simquery" )
local abilityutil = include( "sim/abilities/abilityutil" )

local function neptuneTagTargets(sim, unit )
	if not unit:getBrain():getTarget() then
		return simdefs.BSTATE_FAILED
	end
	
	local weapon = simquery.getEquippedGun( unit )
	
	if not weapon then
		return simdefs.BSTATE_FAILED
	end
	
	local startedShooting = false
	
	while true do
		local shotTarget = false
		for i, target in pairs(unit:getSeenUnits()) do
			if unit:canAct() and simquery.isEnemyAgent( sim:getNPC(), target ) and not target:isDown() and ((target:getTraits().neptuneTag or 0) < (weapon:getTraits().neptuneTagDuration or 0) or (target:getTraits().neptuneMpReduction or 0) < (weapon:getTraits().neptuneMpReduce or 0)) then
				local x0,y0 = unit:getLocation()
				local x1,y1 = target:getLocation()

				local oldFacing = unit:getFacing()
				local newFacing = simquery.getDirectionFromDelta(x1-x0,y1-y0)

				if target:getPlayerOwner() then
					target:getPlayerOwner():glimpseUnit( sim, unit:getID() )
				end

				simquery.suggestAgentFacing(unit, newFacing)
				
				--if not startedShooting then
					startedShooting = true
					sim:dispatchEvent( simdefs.EV_UNIT_START_SHOOTING, { unitID = unit:getID(), newFacing=newFacing, oldFacing=oldFacing,targetUnitID = target:getID() } )
				--end

				if unit:canAct() then
					shotTarget = true
					sim:startTrackerQueue(true)				
					local dmgt = abilityutil.createShotDamage( weapon, unit )
					
					unit:setInvisible( false )
					unit:setDisguise(false,nil,true)
					
					local x0,y0 = unit:getLocation()
					local x1,y1 = target:getLocation()
					local evData = { unitID = unit:getID(), x0 = x0, y0 = y0, x1=x1, y1=y1, dmgt = dmgt } 	
					if x0 == x1 and y0 == y1 then
						evData.facing = target:getFacing()
					end
					
					sim:dispatchEvent( simdefs.EV_UNIT_SHOT, evData )
					
					if sim and not sim.neptune_triggers then
						sim.neptune_triggers = neptune_createTriggers( sim )
					end
					
					local t = target:getTraits().neptuneTag or 0
					if t < (weapon:getTraits().neptuneTagDuration or 0) then
						if t < 1 then
							table.insert(sim.neptuneTaggedAgents, target)
						end
						target:getTraits().neptuneTag = weapon:getTraits().neptuneTagDuration
						sim:dispatchEvent(simdefs.EV_UNIT_FLOAT_TXT,{txt= STRINGS.MOD_CORPS.UI.SEEKER_TAG,x=x1,y=y1,unit = target,color={r=1,g=0.2,b=0.1,a=1}})
					end
					
					if (target:getTraits().neptuneMpReduction or 0) < (weapon:getTraits().neptuneMpReduce or 0) then
						local delta = (target:getTraits().neptuneMpReduction or 0) - weapon:getTraits().neptuneMpReduce
						target:addMPMax( delta )
						target:addMP( delta )
						target:getTraits().neptuneMpReduction = weapon:getTraits().neptuneMpReduce
						sim:dispatchEvent(simdefs.EV_UNIT_FLOAT_TXT,{txt= STRINGS.MOD_CORPS.UI.MP_DRAIN,x=x1,y=y1,unit = target,color={r=2/3,g=1/2,b=1,a=1}})
					end
					
					target:interruptMove( sim )
					sim:triggerEvent( simdefs.TRG_UNIT_SHOT, { targetUnit = target, sourceUnit = unit } )
					sim:processReactions(unit)
					
					sim:startTrackerQueue(false)				
				end
				
				break
			end
		end
		if not shotTarget and unit:canAct() then
			if startedShooting then
				sim:dispatchEvent( simdefs.EV_UNIT_STOP_SHOOTING, { unitID = unit:getID(), facing=unit:getFacing()} )
			end
			return simdefs.BSTATE_COMPLETE
		elseif not shotTarget then
			return simdefs.BSTATE_COMPLETE
		end
	end
	
	return simdefs.BSTATE_COMPLETE
end

Actions.neptuneTagTargets = neptuneTagTargets