local util = include( "modules/util" )


local function init( modApi )
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
	
	modApi.requirements = {"Sim Constructor"}
	
	rawset(_G,"SCRIPT_PATHS",rawget(_G,"SCRIPT_PATHS") or {})
	SCRIPT_PATHS.corp_neptune = scriptPath
	
    KLEIResourceMgr.MountPackage( dataPath .. "/images.kwad", "data" )   
    KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )     
    KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )   
	
	KLEIResourceMgr.MountPackage( dataPath .. "/neptune_partialremix.kwad", "data" )
    MOAIFmodDesigner.loadFEV("neptune.fev")
	
	modApi:addGenerationOption("neptune",STRINGS.MOD_CORPS.OPTIONS.NEPTUNE,STRINGS.MOD_CORPS.OPTIONS.NEPTUNE_DESC, {noUpdate = true})
	modApi:addGenerationOption("map_offset",STRINGS.MOD_CORPS.OPTIONS.OFFSET,STRINGS.MOD_CORPS.OPTIONS.OFFSET_DESC, {noUpdate = true})
	modApi:addGenerationOption("bonus_missions",STRINGS.MOD_CORPS.OPTIONS.BONUS,STRINGS.MOD_CORPS.OPTIONS.BONUS_DESC, {noUpdate = true,values = {0,1,2,3,4,5},value = 0})
	modApi:addGenerationOption("first_mission",STRINGS.MOD_CORPS.OPTIONS.FIRST,STRINGS.MOD_CORPS.OPTIONS.FIRST_DESC, {noUpdate = true,values = {0,1,2},value = 1,strings = STRINGS.MOD_CORPS.OPTIONS.FIRST_STRINGS})
	modApi:addGenerationOption("neptune_only",STRINGS.MOD_CORPS.OPTIONS.NEPTUNE_ONLY,STRINGS.MOD_CORPS.OPTIONS.NEPTUNE_ONLY_DESC, {noUpdate = true,enabled = false})

	
    local worldgen = include( scriptPath .. "/worldgen" )
    for i,world in pairs(worldgen.worlds)do
        modApi:addCorpWorld( i, world )
    end
	
	local cdefs = include("client_defs")
	
	for i, zone in pairs(worldgen.zones) do
		table.insert(cdefs.MAPTILES,zone)
	end
	
	include(scriptPath.."/serverdefs")
	include(scriptPath.."/laserbeam")
	include(scriptPath.."/simpressurecore")
	include(scriptPath.."/simpressureplate")
	include(scriptPath.."/simfrostcloud")
	include(scriptPath.."/mainframe_panel")
	include(scriptPath.."/smokerig")
	include(scriptPath.."/simquery")
	include(scriptPath.."/actions")
	include(scriptPath.."/neptuneGuardbrainSeeker")
	include(scriptPath.."/astar_handlers")
	include(scriptPath.."/unitrig")
	include(scriptPath.."/hud")
	include(scriptPath.."/triggers")
	include(scriptPath.."/pressurerig")

	util.tmerge( STRINGS.LOADING_TIPS, STRINGS.MOD_CORPS.LOADING_TIPS )
end

local function initStrings( modApi )
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()

    local DLC_STRINGS = include( scriptPath .. "/strings" )
    modApi:addStrings( dataPath, "MOD_CORPS", DLC_STRINGS )  	
end

local function earlyLoad( modApi, options )
	--There's no need to use earlyUnload, if we're not loaded Neptune corp won't appear anyways
    local scriptPath = modApi:getScriptPath()
	include(scriptPath.."/mainframe_panel")
	local worldgen = include( scriptPath .. "/worldgen" )
	worldgen.resetWorldGen()
end
		
local function removeNonNeptune(t)
	for k = #t, 1, -1 do
		local tag = t[k]
		if tag == "ftm" or tag == "ko" or tag == "plastech" or tag == "sankaku" then
			table.remove(t,k)
		end
	end
end

local function load( modApi, options, params )
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
	
	local animdefs = include( scriptPath .. "/animdefs" )
    for name, animDef in pairs(animdefs) do
        modApi:addAnimDef( name, animDef )
    end
	
	local propdefs = include( scriptPath .. "/propdefs" )
    for i,item in pairs(propdefs) do
        modApi:addPropDef( i, item, false )
    end
	
	include( scriptPath .. "/itemdefs" )()
    
	local guarddefs = include( scriptPath .. "/guarddefs" )
	for name, guarddef in pairs(guarddefs) do
        modApi:addGuardDef( name, guarddef )
    end
	
    local neptunePrefabs = include( scriptPath .. "/prefabs/neptune/prefabt" )
    modApi:addWorldPrefabt(scriptPath, "neptune", neptunePrefabs)    

    local commondefs = include( scriptPath .. "/commondefs" )
    modApi:addTooltipDef( commondefs )
	
	if options["neptune"].enabled then
		local widget = {
			name = [[antartica]],
			isVisible = true,
			noInput = false,
			anchor = 1,
			rotation = 0,
			x = 0,
			xpx = true,
			y = -139,
			ypx = true,
			w = 3552,
			wpx = true,
			h = 610,
			hpx = true,
			sx = 1,
			sy = 1,
			scissor =
			{
				-0.166666001081467,
				-0.5,
				0.166633993387222,
				0.5,
			},
			ctor = [[image]],
			color =
			{
				1,
				1,
				1,
				1,
			},
			tilew = 3,
			tileh = 1,
			images =
			{
				{
					file = [[gui/map_antartica.png]],
					name = [[]],
				}
			}
		}
		local map_colour_normal = {61/255,13/255,47/255,1}
		local map_colour_highlight = {166/255,66/255,206/255,1}
		local map_colour_unhighlight =  {34/255,57/255,56/255,.5}
		modApi:addCorpMap( "neptune", widget, map_colour_normal, map_colour_highlight, map_colour_unhighlight, true )
		
	
		if params and options["bonus_missions"] and options["bonus_missions"].value > 0 then
			local check = {
				stage = 2,
				fn = function( campaign, hours, stage )
					if campaign.hours < campaign.difficultyOptions.maxHours and (campaign.currentDay > campaign.previousDay or (campaign.hours > 0 and not campaign.neptune_missions_spawned)) then
						campaign.neptune_missions_spawned = true
						local serverdefs = include("modules/serverdefs")
						local simdefs = include("sim/simdefs")
						local tags = util.tmerge( { "2max", "neptune" }, simdefs.ESCAPE_MISSION_TAGS )
						serverdefs.createCampaignSituations( campaign, options["bonus_missions"].value, tags )
					end
				end,
			}
			modApi:addAdvanceTimeCheck(check)
		end
		
		local serverdefs = include("modules/serverdefs")
		if (options["first_mission"] and options["first_mission"].value ~= 0) or (options["neptune_only"] and options["neptune_only"].enabled) then
			table.insert(serverdefs.MISSION_TAGS_INITIAL_EXEC,"neptune")
		end
		if (options["first_mission"] and options["first_mission"].value == 2) or (options["neptune_only"] and options["neptune_only"].enabled) then
			removeNonNeptune(serverdefs.MISSION_TAGS_INITIAL_EXEC)
		end
		
		if options["neptune_only"] and options["neptune_only"].enabled then
			local simdefs = include("sim/simdefs")
			removeNonNeptune(serverdefs.CORP_NAMES)
			removeNonNeptune(simdefs.ESCAPE_MISSION_TAGS)
			removeNonNeptune(simdefs.NO_CFO_MISSION_TAGS)
			removeNonNeptune(simdefs.DEFAULT_MISSION_TAGS)
			removeNonNeptune(simdefs.DEFAULT_MISSION_TAGS_FIRST)
		end
	end
	
	if options["map_offset"] and options["map_offset"].enabled then
		local simdefs = include("sim/simdefs")
		simdefs.map_offset.y = 100
	end
	
	local passives = include( scriptPath .. "/passive_abilities" )
	local passive_abilities = include( "sim/abilities/passive_abilities" )
	for name, passive in pairs(passives) do
		passive_abilities[name] = passive
	end
end

return {
    init = init,
    load = load,
	earlyLoad = earlyLoad,
    initStrings = initStrings,
}