local array = include( "modules/array" )
local serverdefs = include( "modules/serverdefs" )
local mathutil = include( "modules/mathutil" )
local array = include( "modules/array" )
local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local simplayer = include("sim/simplayer")
local unitdefs = include( "sim/unitdefs" )
local worldgen = include( "sim/worldgen" )
local simfactory = include( "sim/simfactory" )

-------------------------------------------------------------------------------
-- These are passive abilities (no executeAbility function)

local function increaseIce(unit,sim,iceInc)
	if unit:getTraits().mainframe_iceMax then
		sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = unit, ice = unit:getTraits().mainframe_ice, delta = iceInc } )

		local raiseIce = true
		if unit:getTraits().noRaiseIfIceZero and unit:getTraits().mainframe_ice <= 0 then
			raiseIce = false
		end

		if unit:getTraits().mainframe_iceMax and not unit:getTraits().mainframe_console and ( not unit:getPlayerOwner() or not unit:getPlayerOwner():isPC() ) then

			if raiseIce then
				unit:getTraits().mainframe_ice = unit:getTraits().mainframe_ice+iceInc
				sim:triggerEvent( simdefs.TRG_ICE_BROKEN, { unit = unit, delta = iceInc } )
			end

			unit:getTraits().mainframe_iceMax = unit:getTraits().mainframe_iceMax+iceInc					
		end
	end
end

local function formatToolTip( header, body )
	return string.format( "<ttheader>%s\n<ttbody>%s</>", util.toupper(header), body )
end

local DEFAULT_PASSIVE =
{
	getName = function( self, sim, unit )
		return self.name
	end,
		
	createToolTip = function( self,sim,unit,targetUnit)
		return formatToolTip( self.name, string.format("PASSIVE\n%s", self.desc ) )
	end,

	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	
	ghostable = true,

	executeAbility = nil, -- Passives by definition have no execute.
}

local DEFAULT_DEBUFF =
{
	debuffAbility = true, 

	getName = function( self, sim, unit )
		return self.name
	end,
		
	createToolTip = function( self,sim,unit,targetUnit)
		return formatToolTip( self.name, string.format("DEBUFF\n%s", self.desc ) )
	end,

	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	
	ghostable = true,

	executeAbility = nil, -- Passives by definition have no execute.
}

local DEFAULT_BUFF =
{
	buffAbility = true, 

	getName = function( self, sim, unit )
		return self.name
	end,
		
	createToolTip = function( self,sim,unit,targetUnit)
		return formatToolTip( self.name, string.format("BUFF\n%s", self.desc ) )
	end,

	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	
	ghostable = true,

	executeAbility = nil, -- Passives by definition have no execute.
}

local passive_abilities =
{
	neptune_frost_zone = util.extend( DEFAULT_BUFF )
	{
		buffAbility = false,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )				
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			local x0, y0 = userUnit:getLocation()
			if x0 and evData:isPC() and not userUnit:isDown() and sim:getTurnCount() > 0 then
				local cells = simquery.fillCircle( sim, x0, y0, userUnit:getTraits().frostZoneRange, 0 )
				for i, cell in pairs(cells) do
					for i, unit in pairs(cell.units) do
						if simquery.isEnemyAgent( sim:getNPC(), unit, true) and unit:getTraits().mp > 5 then
							unit:getTraits().mp = math.max(unit:getTraits().mp - userUnit:getTraits().frostZone,5)
							sim:dispatchEvent(simdefs.EV_UNIT_FLOAT_TXT,{txt= STRINGS.MOD_CORPS.UI.FROST_ZONE,x=cell.x,y=cell.y,unit = unit,color={r=1/2,g=1,b=1,a=1}})
							sim:dispatchEvent( simdefs.EV_HUD_MPUSED, unit )
						end
					end
				end
				sim:dispatchEvent("EV_CE_HILITE_RADIUS",{x=x0,y=y0,range=userUnit:getTraits().frostZoneRange,cells = cells, color = { 0.5, 1, 1, 0.5 },rate = {8, 30, 0}})
			end
		end, 
	},
}

return passive_abilities