local util = include("modules/util")
local mui_util = include("mui/mui_util")
local commondefs = include("sim/unitdefs/commondefs")

local propdefs = {
	pressure_core = {
		type = "simpressurecore",
		name = STRINGS.MOD_CORPS.PROPS.PRESSURE_CORE,
		kanim = "corp_kanim_pressurecore",
		onWorldTooltip = commondefs.onMainframeTooltip,
		rig = "pressurerig",
		traits = util.extend( commondefs.MAINFRAME_TRAITS )
			{
				pressureAreaRange = 3,
				plateType = "pressure_plate",
				recap_icon = "sound_bug",
			},
		sounds = {appeared="SpySociety/HUD/gameplay/peek_negative",reboot_start="SpySociety/Actions/reboot_initiated_camera",reboot_end="SpySociety/Actions/reboot_complete_camera"},		
	},
	
	pressure_plate = {
		type = "simpressureplate",
		name = STRINGS.MOD_CORPS.PROPS.PRESSURE_PLATE,
		kanim = "corp_kanim_pressureplate",
		
		onWorldTooltip = function( tooltip, unit )
			commondefs.onMainframeTooltip( tooltip, unit )
			local binding = util.getKeyBinding( "mainframeMode" )
			if binding then
				tooltip:appendHeader( mui_util.getBindingName( binding ), STRINGS.UI.HUD_MAINFRAME )
			end
		end,
		
		rig = "pressurerig",
		traits = {
			--mainframe_item = true,
			mainframe_item = false,
			unhackable_mainframe_item = true,
			mainframe_icon = true,
			mainframe_no_daemon_spawn = true,
			mainframe_status = "active",
		},
		sounds = {appeared="SpySociety/HUD/gameplay/peek_negative",reboot_start="SpySociety/Actions/reboot_initiated_camera",reboot_end="SpySociety/Actions/reboot_complete_camera"},		
	},
	
	standalone_pressure_plate = {
		type = "simpressureplate",
		name = STRINGS.MOD_CORPS.PROPS.PRESSURE_PLATE,
		kanim = "corp_kanim_pressureplate",
		onWorldTooltip = commondefs.onMainframeTooltip,
		rig = "corerig",
		traits = {
			mainframe_item = true,
			mainframe_icon = true,
			mainframe_status = "active",
			mainframe_ice = 1,
			mainframe_iceMax = 1,
		},
		sounds = {appeared="SpySociety/HUD/gameplay/peek_negative",reboot_start="SpySociety/Actions/reboot_initiated_camera",reboot_end="SpySociety/Actions/reboot_complete_camera"},		
	},
	
	frost_cloud =
	{
		type = "simfrostcloud",
		name = STRINGS.PROPS.SMOKE,
		rig = "smokerig",
		kanim = "kanim_smoke_plume",
		traits = { radius = 4, lifetime = 5, noghost = true, gasColor = {r = 0.5, g = 1, b = 1, a = 0.25} }
	},
	
	NEPTUNE_security_infrared_wall_emitter_1x1 =
	{
		type = "laser_emitter",
		name =  STRINGS.MOD_CORPS.PROPS.WALL_EMITTER,
		kanim = "kanim_wall_laser_emitter_tall",
		onWorldTooltip = commondefs.onMainframeTooltip,
		--facing = simdefs.DIR_N,
		rig = "laserrig",
		traits =
		{
			empWhenTripped = 1,
			mainframe_icon = true,
			mainframe_laser=true,
			mainframe_spawnprop = "neptune_lasergrid",
			mainframe_spawnpartner = "security_laser_emitter_tall_1x1_partner",
			mainframe_status = "inactive",
			mainframe_autodeactivate = true,
		},
		sounds = {appeared="SpySociety/HUD/gameplay/peek_negative"},
	},	

	neptune_lasergrid =
	{
		type = "neptunebeam",
		name = STRINGS.MOD_CORPS.PROPS.INFRARED_WALL, 
		kanim = "kanim_neptune_lasergrid",
		--facing = simdefs.DIR_N,
		traits = { noghost = true, toolTipNote = STRINGS.MOD_CORPS.PROPS.INFRARED_WALL_TOOLTIP, mainframe_icon = true, redSymbols={"red","wallbeam_red","wallBeamSymbol_red","lines_new"},tealSymbols={"teal","wallbeam","wallBeamSymbol","lines_new_teal"}},
		onWorldTooltip = commondefs.onBeamTooltip,
		rig = "beamrig",		
	},
}

for i, prop in pairs(propdefs) do
	prop.id = i
	prop.k = i
end

return propdefs