----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local simfactory = include( "sim/simfactory" )
local unitdefs = include("sim/unitdefs")
local cdefs = include( "client_defs" )

-----------------------------------------------------
-- Local functions

local simpressurecore =
{
    ClassType = "simpressurecore",
}

function simpressurecore:activate( sim )
	if self:getTraits().mainframe_status == "inactive" then
		self:getTraits().mainframe_status = "active"
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
		for i, unit in pairs(self:getTraits().pressureplates) do
			if not unit:getTraits().mainframe_booting then
				unit:getTraits().mainframe_status = "inactive"
				unit:activate( sim )
				sim:getPC():glimpseUnit(sim,unit:getID())
			end
		end
	end
end

function simpressurecore:deactivate( sim )	
	if self:getTraits().mainframe_status == "active" then
		self:getTraits().mainframe_status = "inactive"
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
		for i, unit in pairs(self:getTraits().pressureplates) do
			unit:deactivate( sim )
			sim:getPC():glimpseUnit(sim,unit:getID())
		end
	end
end

local function pathFn( cellquery, unit, startcell, endcell )
	if endcell == nil or endcell.sightblock then
		return false
	end

	local dx, dy = endcell.x - startcell.x, endcell.y - startcell.y
	if (startcell.x + 1 == endcell.x or startcell.x - 1 == endcell.x) and (startcell.y + 1 == endcell.y or startcell.y - 1 == endcell.y) then
		-- diagonal
		local cell3 = cellquery:getCell( startcell.x, endcell.y )
		local cell4 = cellquery:getCell( endcell.x, startcell.y )
		if cell3 == nil or cell4 == nil then
			return false
		end
		local d1, d2 = simquery.getDirectionFromDelta( cell3.x - startcell.x, cell3.y - startcell.y ), simquery.getDirectionFromDelta( endcell.x - cell3.x, endcell.y - cell3.y )
		local d3, d4 = simquery.getDirectionFromDelta( cell4.x - startcell.x, cell4.y - startcell.y ), simquery.getDirectionFromDelta( endcell.x - cell4.x, endcell.y - cell4.y )

		if not ((simquery.isOpenExit( startcell.exits[ d1 ] ) and simquery.isOpenExit( cell3.exits[ d2 ] )) or
				(simquery.isOpenExit( startcell.exits[ d3 ] ) and simquery.isOpenExit( cell4.exits[ d4 ] ))) then
			return false
		end
	
	else
		local dir = simquery.getDirectionFromDelta( dx, dy )
		if not simquery.isOpenExit( startcell.exits[ dir ] ) then
			return false
		end
	end
	
	return true
end

local function costFn( cell1, cell2 )
	return math.max(math.abs(cell1.x - cell2.x),math.abs(cell1.y - cell2.y))
end

local function canPutPlate(cell)
	if cell.impass and cell.impass > 0 then
		return false
	end
	for i, unit in pairs(cell.units) do
		if unit:getTraits().pressureID or unit:getTraits().pressureplates then
			return false
		end
	end
	return true
end

function simpressurecore:onWarp( sim, oldcell, cell)
	if not oldcell and cell then
		if not sim.neptune_triggers then
			sim.neptune_triggers = neptune_createTriggers( sim )
		end
		
		self:getTraits().pressureplates = {}
		local cells = simquery.floodFill( sim, nil, cell, self:getTraits().pressureAreaRange, costFn, pathFn )
		for i, cell in pairs(cells) do
			if canPutPlate(cell) then
				local unitData = unitdefs.lookupTemplate( self:getTraits().plateType )
				local newUnit = simfactory.createUnit( unitData, sim )
				newUnit:setPlayerOwner( self:getPlayerOwner() )
				newUnit:getTraits().pressureID = self:getID()
				table.insert(self:getTraits().pressureplates,newUnit)
				sim:spawnUnit( newUnit )
				sim:warpUnit( newUnit, cell )
			end
		end
	elseif not cell and oldcell then
		for i, unit in pairs(self:getTraits().pressureplates) do
			unit:deactivate( sim )
			sim:warpUnit( unit, nil )
			sim:despawnUnit( unit )
		end
	end
end

function simpressurecore:setPlayerOwner( player )
    simunit.setPlayerOwner( self, player )
	for i, unit in pairs(self:getTraits().pressureplates) do
		unit:setPlayerOwner( player )
		self._sim:getPC():glimpseUnit(self._sim,unit:getID())
		self._sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit} )
	end
end

-----------------------------------------------------
-- Interface functions

local function createPressurecore( unitData, sim )
	return simunit.createUnit( unitData, sim, simpressurecore )
end

simfactory.register( createPressurecore )

return simpressurecore
