----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local simfactory = include( "sim/simfactory" )
local cdefs = include( "client_defs" )

-----------------------------------------------------
-- Local functions

local simpressureplate =
{
	ClassType = "simpressureplate",
}

function simpressureplate:getTracker()
	return self:getTraits().pressureID and self._sim:getUnit(self:getTraits().pressureID) or self
end

function simpressureplate:refreshTracking()
	local sim = self._sim
	local x,y = self:getLocation()
	local cell = sim:getCell(x,y)
	if cell then
		for i, targetUnit in ipairs( cell.units ) do
			self:onUnitStep( sim, targetUnit )
		end
	end
end

function simpressureplate:performTracking( sim, seenUnit )
	local tracker = self:getTracker()
	
	if seenUnit:getPlayerOwner() then
        seenUnit:getPlayerOwner():glimpseUnit( sim, tracker:getID() )
    end 
	
    seenUnit:interruptMove( sim, tracker )
    -- Flag that we tracked already this turn.
    tracker:getTraits().tracker_alert = sim:getTurnCount()
	
	sim:dispatchEvent( simdefs.EV_CAM_PAN, { seenUnit:getLocation() } )	
	sim:dispatchEvent( simdefs.EV_UNIT_RESET_ANIM_PLAYBACK, { unit = seenUnit } )

	local x0, y0 = tracker:getLocation()
	sim:emitSound( simdefs.SOUND_SECURITY_ALERTED, x0, y0, tracker )
	local x1, y1 = seenUnit:getLocation()
	sim:triggerEvent( simdefs.TRG_NEW_INTEREST, { x = x0, y = y0, range = simdefs.SOUND_RANGE_3, interest = { x= x1, y = y1, reason=simdefs.REASON_CAMERA} })

	sim:dispatchEvent( simdefs.EV_UNIT_ALERTED, { unitID = tracker:getID() } )

	sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(  STRINGS.UI.ALARM_ADD, 1 ),x=x0,y=y0,color={r=255/255,g=10/255,b=10/255,a=1}, alwaysShow = true} )
	sim:trackerAdvance( 1, STRINGS.MOD_CORPS.UI.ALERT_PRESSURE )
	sim:triggerEvent( simdefs.TRG_CAUGHT_BY_CAMERA)

end

function simpressureplate:activate( sim )
	if (self:getTraits().pressureID == nil or sim:getUnit(self:getTraits().pressureID):getTraits().mainframe_status == "active") and self:getTraits().mainframe_status == "inactive" then
		self:getTraits().mainframe_status = "active"
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
		self:refreshTracking()
	end
end

function simpressureplate:setPlayerOwner(player)
	simunit.setPlayerOwner(self,player)
	self:refreshTracking()
end

function simpressureplate:deactivate( sim )
	if self:getTraits().mainframe_status == "active" then
		self:getTraits().mainframe_status = "off"
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
	end
end

function simpressureplate:canTrackTarget( sim, targetUnit )
	if not self:isPC() and self:getTracker():getTraits().mainframe_status == "active" and ( (self:getTracker():getTraits().tracker_alert or -1) < sim:getTurnCount() ) then
		if targetUnit and simquery.isEnemyAgent( self:getPlayerOwner(), targetUnit ) and not targetUnit:isKO() and targetUnit:getTraits().dynamicImpass then
			return true
		end
	end
	return false
end

function simpressureplate:canGlimpseTarget( sim, targetUnit )
	if self:isPC() and targetUnit and simquery.isEnemyAgent( self:getPlayerOwner(), targetUnit ) and self:getTracker():getTraits().mainframe_status == "active" and not targetUnit:isKO() and targetUnit:getTraits().dynamicImpass then
		return true
	end
	return false
end

function simpressureplate:onUnitStep( sim, unit )
	if self:getTraits().mainframe_status ~= "active" then
		return
	end
	
	if self:canTrackTarget( sim, unit ) then
		self:performTracking( sim, unit )
	elseif self:canGlimpseTarget( sim, unit ) then
		sim:getPC():glimpseUnit( sim, unit:getID() )
	end
end

-----------------------------------------------------
-- Interface functions

local function createPressureplate( unitData, sim )
	return simunit.createUnit( unitData, sim, simpressureplate )
end

simfactory.register( createPressureplate )

return simpressureplate
