local simdefs = include("sim/simdefs")
local util = include("modules/util")
local mathutil = include("modules/mathutil")
local simquery = include("sim/simquery")

local triggerObject = {
	onTrigger = function( self, sim, evType, evData )
		if evType == simdefs.TRG_UNIT_WARP_PRE then
			-- REMOVE AP FROM AGENTS MOVING THROUGH FROST CLOUDS --
			
			local moveCost = nil
			local unit = evData.unit
			local baseCost = simquery.getTrueMoveCost( unit, evData.from_cell, evData.to_cell ) - simquery.getMoveCost( evData.from_cell, evData.to_cell )
			if unit:getTraits().movingBody then

				local dragCost = simdefs.DRAGGING_COST

				if unit:hasTrait("dragCostMod") then
					dragCost = dragCost -  unit:getTraits().dragCostMod
				end

				--easier to drag agents
				local body = unit:getTraits().movingBody
				if not body:getTraits().isGuard then
					dragCost = dragCost - 1
				end

				--never improve moving if you're dragging!
				dragCost = math.max(simdefs.MIN_DRAGGING_COST, dragCost)

				moveCost = baseCost * dragCost
			elseif unit:getTraits().sneaking then
				moveCost = baseCost * simdefs.SNEAKING_COST
			else 
				moveCost = baseCost
			end
			unit:useMP(moveCost, sim )
		elseif evType == simdefs.TRG_START_TURN then
			-- TRACK AGENTS TAGGED BY SEEKERS --
			
			if evData:isPC() and #sim.neptuneTaggedAgents > 0 then
				local eligibleGuards = {}
			
				for i, unit in pairs(sim:getNPC():getUnits()) do
					if unit:getBrain() and not unit:isKO() and not unit:getTraits().camera_drone
						and not unit:getBrain():getTarget() then
						
						local hasInterestAlready = false
						local interest = unit:getBrain():getInterest()
						
						if interest then
							local cell = sim:getCell(interest.x, interest.y)
							if cell then
								for i, unit in pairs(cell.units) do
									if unit:getTraits().neptuneTag then
										hasInterestAlready = true
									end
								end
							end
						end
						
						if not hasInterestAlready then
							table.insert(eligibleGuards, unit)
						end
					end
				end
				
				local i = 1
				while i <= #sim.neptuneTaggedAgents do
					local agent = sim.neptuneTaggedAgents[i]
					agent:getTraits().neptuneTag = agent:getTraits().neptuneTag - 1
					if agent:getTraits().neptuneTag > 0 then
						i = i + 1
					else
						agent:getTraits().neptuneTag = nil
						table.remove(sim.neptuneTaggedAgents,i)
					end
					local x0, y0 = agent:getLocation()
					
					if x0 and not agent:isDead() then
						local nearestGuard = nil
						local bestDist = math.huge
						
						for k, unit in ipairs(eligibleGuards) do
							local x1, y1 = unit:getLocation()
							if x1 then
								local dist = mathutil.distSqr2d( x0, y0, x1, y1 )
								if dist < bestDist then
									bestDist = dist
									nearestGuard = k
								end
							end
						end
						
						if nearestGuard then
							nearestGuard = table.remove( eligibleGuards, nearestGuard )
							nearestGuard:getBrain():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_CAMERA, agent)
							sim:dispatchEvent(simdefs.EV_UNIT_FLOAT_TXT,{txt= STRINGS.MOD_CORPS.UI.SEEKER_TAG,x=x0,y=y0,unit = agent,color={r=1,g=0.2,b=0.1,a=1}})
							sim:dispatchEvent( simdefs.EV_UNIT_WIRELESS_SCAN, { unitID = agent:getID() } )
						end
					end
				end
			end
		elseif evType == simdefs.TRG_OVERWATCH then
			-- PRESSURE SENSOR TRACKING
			
			if evData then
				local x,y = evData:getLocation()
				local cell = sim:getCell(x,y)
				if cell then
					for i, sensor in ipairs( cell.units ) do
						if sensor.onUnitStep then
							sensor:onUnitStep( sim, evData )
						end
					end
				end
			end
		end
	end
}

function neptune_createTriggers( sim )
	local self = util.tcopy(triggerObject)
	sim.neptuneTaggedAgents = {}
	sim:addTrigger(simdefs.TRG_UNIT_WARP_PRE,self)
	sim:addTrigger(simdefs.TRG_START_TURN,self)
	sim:addTrigger(simdefs.TRG_OVERWATCH,self)
	return self
end