local serverdefs = include( "modules/serverdefs" )
local util = include( "modules/util" )
local weighted_list = include( "modules/weighted_list" )
local mathutil = include( "modules/mathutil" )
local array = include( "modules/array" )
local rand = include( "modules/rand" )
local cdefs = include( "client_defs" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local mazegen = include( "sim/mazegen" )
local roomgen = include( "sim/roomgen" )
local prefabs = include( "sim/prefabs" )
local unitdefs = include( "sim/unitdefs" )
local procgen_context = include( "sim/procgen_context" )
local npc_abilities = include( "sim/abilities/npc_abilities" )
local version = include( "modules/version" )
local worldgen = include( "sim/worldgen" )

if worldgen.generateSideMission == nil then
	
	worldgen.generateSideMission = function( params, cxt )
		local rand = rand.createGenerator( params.seed )
		
		local i = rand:nextInt(1,100)

		if params.missionCount > 0 and version.isVersionOrHigher( params.missionVersion or "", "0.17.9") and i < (rawget(_G,"SIDEMISSIONODDS") or 25) and #worldgen.SIDEMISSIONS > 0 then --25
			params.side_mission = worldgen.SIDEMISSIONS[rand:nextInt(1,#worldgen.SIDEMISSIONS)]
		end
	end
end

worldgen.NEPTUNE={
    THREAT_DEFAULT = { {"neptune_guard_tier_2",100}, {"neptune_frost_guard",100}, {"neptune_seeker_guard",100} },
    GUARD_DEFAULT = { {"neptune_guard",100}, {"neptune_guard_weak",100} },
    LVL2_DEFAULT = { {"neptune_frost_guard_LV2",400}, {"neptune_seeker_guard_LV2",400} },
	ADMIN_DEFAULT = { {"neptune_sysadmin",100} },
	ADMIN_NEW_DEFAULT = { {"neptune_sysadmin_new",100} },
	
    THREAT = {},
    GUARD = {},
    LVL2 ={},
	ADMIN = {},
	ADMIN_NEW = {},
	
    PREFABS_DEFAULT = {},
    PREFABS = {},
}

local function UVCustomTex( left, bottom, right, top )
	local TOTALW, TOTALH = 2048, 2048 -- Total texture size of cdefs.WALLTILES_FILE
	return { (left+0.5) / TOTALW, (bottom+0.5) / TOTALH, (right+0.5) / TOTALW, (top+0.5) / TOTALH }
end

local function UVTex( rowx, rowy )
	local CELLW, CELLH = 256, 256 -- Tile size in cdefs.WALLTILES_FILE
	local left, top = rowx * CELLW, rowy * CELLH
	local right, bottom = (rowx + 1) * CELLW - 1, (rowy + 1) * CELLH - 1
	return UVCustomTex( left, bottom, right, top )
end

local function UVPostTex( rowx, rowy )
	local CELLW, CELLH = 256, 256 -- Tile size in cdefs.WALLTILES_FILE
	local left, top = rowx * CELLW, (rowy + 1) * CELLH - 9
	local right, bottom = (rowx + 1) * CELLW - 1, (rowy + 1) * CELLH - 1
	return UVCustomTex( left, bottom, right, top )
end

local ZONE_ENGINE = { name = "neptune_engine", pattern = "engine", wallUV = UVTex( 0, 0 ), texFile = "data/images/corp_mod/walls.png" }
local ZONE_OFFICE = { name = "neptune_office", pattern = "office", wallUV = UVTex( 1, 0 ), texFile = "data/images/corp_mod/walls.png" }
local ZONE_FREEZER = { name = "neptune_freezer", pattern = "freezer", wallUV = UVTex( 2, 0 ), texFile = "data/images/corp_mod/walls.png" }
local ZONE_HALL = { name = "neptune_hall", pattern = "hall", wallUV = UVTex( 3, 0 ), texFile = "data/images/corp_mod/walls.png" }

local ZONES = {
	{
		zone = ZONE_FREEZER, variant = 0,
		tileStart = 10*21+1, patternLen = 3, 
		render_filter = { fow = "zone_1_fog", normal = "zone_1", dynamic = "zone_1_dynamic" },
		ambientSound = 1,
	},	
	
	{
		zone = ZONE_ENGINE, variant = 0,
		tileStart = 10*21+7, patternLen = 3, 
		render_filter = { fow = "zone_1_fog", normal = "zone_1", dynamic = "zone_1_dynamic" },
		ambientSound = 1,
	},

	{
		zone = ZONE_OFFICE, variant = 0,
		tileStart = 10*21+4, patternLen = 3, 
		render_filter = { fow = "zone_1_fog", normal = "zone_1", dynamic = "zone_1_dynamic" },
		ambientSound = 1,
	},	
	
	{
		zone = ZONE_HALL, variant = 0,
		tileStart = 10*21+10, patternLen = 3, 
		render_filter = { fow = "zone_1_fog", normal = "zone_1", dynamic = "zone_1_dynamic" },
		ambientSound = 1,
	},	
}

cdefs.LEVELTILES_PARAMS =
{
	file = "data/images/corp_mod/leveltiles.png",
	21,			--width in tiles
	21,			--height in tiles
	48/1008,		--cellWidth
	48/1008,		--cellHeight
	0.5/1008,	--xOffset
	0.5/1008,	--yOffset
	47/1008,		--tileWidth
	47/1008,		--tileHeight
}

local neptune = class( procgen_context )

function neptune:init( ... )
	procgen_context.init( self, ... )

	self.ZONES = { ZONE_ENGINE,ZONE_OFFICE, ZONE_FREEZER }	
	self.HALL_ZONE =  ZONE_HALL

    worldgen.generateSideMission( self.params, self._cxt )
end

function neptune:generatePrefabs( )
	local candidates = {}
	
	self.NUM_CAMERAS = worldgen.getCameraNumbers(self.params,self.difficulty) - math.ceil(math.min(self.difficulty,4) * 0.5)
	
	prefabs.generatePrefabs( self, candidates, "entry_guard", 2 )
	prefabs.generatePrefabs( self, candidates, "barrier", math.floor( self.NUM_BARRIERS/2 ) )
	prefabs.generatePrefabs( self, candidates, "console", self.NUM_CONSOLES )
	prefabs.generatePrefabs( self, candidates, "null_console", self.NUM_NULL_CONSOLES )
	
	if self.difficulty >= 3 then
		prefabs.generatePrefabs( self, candidates, "frost_spire", math.max(1,math.floor(self.difficulty/5)) )
	end
    
	worldgen.generateMainframes( self, candidates )
    worldgen.generateMiniServers( self, candidates )
	
	prefabs.generatePrefabs( self, candidates, "store", 1 )
	prefabs.generatePrefabs( self, candidates, "safe", self.NUM_SAFES, worldgen.lootFitness )
	prefabs.generatePrefabs( self, candidates, "secret", 2 )
	prefabs.generatePrefabs( self, candidates, "camera", self.NUM_CAMERAS, worldgen.cameraFitness( candidates ) )
	prefabs.generatePrefabs( self, candidates, "decor")

    worldgen.generateLaserGenerators( self, candidates )

	return candidates
end

function neptune:generateUnit( unit )
    unit.template = unit.template:gsub( "security_laser_emitter_1x1", "NEPTUNE_security_infrared_wall_emitter_1x1" )
    return unit
end

function neptune:generateUnits()

    local SPAWN_TABLE = {
        COMMON = worldgen.NEPTUNE.GUARD,
        ELITE = worldgen.NEPTUNE.THREAT,
        CAMERA_DRONE = worldgen.CAMERA_DRONE_FIX,
        OMNI = worldgen.NEPTUNE.LVL2,
		SYS_ADMIN = worldgen.NEPTUNE.ADMIN,
		SYS_ADMIN_NEW = worldgen.NEPTUNE.ADMIN_NEW,
    }

    self._patrolGuard = worldgen.NEPTUNE.GUARD

    worldgen.generateThreats( self, SPAWN_TABLE )
	worldgen.generateGuardLoot( self, unitdefs.prop_templates.passcard )
	worldgen.generateGuardLoot( self, unitdefs.prop_templates.passcard )

	local LOCAL_DAEMONS = {}
    for i,program in ipairs(serverdefs.PROGRAM_LIST)do
        LOCAL_DAEMONS[program] = 1
    end
	
	local pressureCores = {}
	local MIN_DISTANCE = 3
	local TEMPLATE_NAME = "pressure_core"
	
	for i = 1, 1 + math.ceil(math.min(self.difficulty,5) * 0.75) do
	    local cells = {}
		for i, room in pairs(self.rooms) do
			if not room.tags.entry then
				for _, rect in ipairs(room.rects) do
					for x = rect.x0, rect.x1 do
						for y = rect.y0, rect.y1 do
							local minThreatDist = math.huge
							for i, threatUnit in ipairs(pressureCores) do
								minThreatDist = math.min( minThreatDist, math.max( math.abs(x - threatUnit.x), math.abs(x - threatUnit.x) ))
							end
							if minThreatDist > MIN_DISTANCE and worldgen.canGuardSpawn( self, x, y ) then
								table.insert( cells, x )
								table.insert( cells, y )
							end
						end
					end
				end
			end
		end
		
		if #cells > 0 then
			local i = self.rnd:nextInt( 1, #cells / 2 )
			local unit = {
				x = cells[2 * i - 1],
				y = cells[2 * i],
				template = TEMPLATE_NAME,
			}
			table.insert( pressureCores, unit )
			table.insert( self.units, unit )
		elseif MIN_DISTANCE > 0 then
			MIN_DISTANCE = 0
		else
			break
		end
	end
	
    self.ice_programs = util.weighted_list( LOCAL_DAEMONS )
end

local function resetWorldGen()
	util.tclear(worldgen.NEPTUNE.THREAT)
	util.tmerge(worldgen.NEPTUNE.THREAT,worldgen.NEPTUNE.THREAT_DEFAULT)
	util.tclear(worldgen.NEPTUNE.GUARD)
	util.tmerge(worldgen.NEPTUNE.GUARD,worldgen.NEPTUNE.GUARD_DEFAULT)
	util.tclear(worldgen.NEPTUNE.ADMIN)
	util.tmerge(worldgen.NEPTUNE.ADMIN,worldgen.NEPTUNE.ADMIN_DEFAULT)
	util.tclear(worldgen.NEPTUNE.ADMIN_NEW)
	util.tmerge(worldgen.NEPTUNE.ADMIN_NEW,worldgen.NEPTUNE.ADMIN_NEW_DEFAULT)
	util.tclear(worldgen.NEPTUNE.LVL2)
	util.tmerge(worldgen.NEPTUNE.LVL2,worldgen.NEPTUNE.LVL2_DEFAULT,worldgen.OMNI_GUARD_FIX)
end

resetWorldGen()

return 
{
    worlds = {
		neptune = neptune,
	},
	zones = ZONES,
	resetWorldGen = resetWorldGen,
}